import { NextFunction, Response } from 'express';
import { ErrorMiddleware } from '.';
import { HttpException } from '@exceptions/http.exception';
import { HttpStatusCode } from '@constants/http.enum';
import { ApiService } from 'types/http';
import { DataPayload, RequestWithUser } from '@interfaces/auth.interface';
import jwt from 'jsonwebtoken';
import { KINERJA_PUBLIC_KEY } from '@config';
import { UserService } from '@services/user.service';

class AuthMiddleware {
  public static async authenticateTokenFromService(req: RequestWithUser, res: Response, next: NextFunction, service: ApiService) {
    try {
      const authHeader = req.headers.authorization;
      const token = authHeader && authHeader.split(' ')[1]; // Bearer Token

      if (!token) {
        throw new HttpException(HttpStatusCode.Unauthorized, 'Invalid credentials');
      }

      let publicKey: string;
      switch (service) {
        case ApiService.KINERJA_SERVICE:
          publicKey = KINERJA_PUBLIC_KEY;
          break;
        default:
          throw new HttpException(HttpStatusCode.InternalServerError, 'Invalid service');
      }
      
      const payload: DataPayload = await new Promise((resolve, reject) => {
        jwt.verify(token, publicKey, 
          { issuer: 'kinerja-server', algorithms: ['RS256'] },
          (err, decoded) => {
          if (err) {
            reject(new HttpException(HttpStatusCode.Unauthorized, 'Invalid credentials', err));
          } else {
            resolve(decoded as DataPayload);
          }
        });
      });

      const userService = new UserService();
      const user = await userService.getUserById(payload.user_id);

      if (!user) {
        throw new HttpException(HttpStatusCode.Unauthorized, 'Invalid credentials');
      }

      if (user.status !== 1) {
        throw new HttpException(HttpStatusCode.Unauthorized, 'Account is not active');
      }

      req.user = user;
      next();
    } catch (error) {
      ErrorMiddleware.handle(res, error);
    }
  }
}

export default AuthMiddleware;
