import { NextFunction, Request, Response } from 'express';
import { AnyZodObject } from 'zod';
import { IRequestResponseHandler } from '@interfaces/http.interface';
import { ErrorMiddleware } from '.';
import { HttpException } from '@exceptions/http.exception';
import { HttpStatusCode } from '@constants/http.enum';

class ValidationMiddleware {
  public static validate(schema: AnyZodObject) {
    return async (req: Request, res: Response, next: NextFunction) => {
      try {
        await schema.parseAsync({
          body: req.body,
          query: req.query,
          params: req.params,
        });
        return next();
      } catch (error) {
        ErrorMiddleware.handle(res, error);
      }
    };
  }

  public static exceptionGuard(handler: IRequestResponseHandler) {
    return async (req: Request, res: Response) => {
      const { email } = req.body;
      try {
        await handler(req, res);
      } catch (error) {
        if (error instanceof HttpException && error.statusCode === HttpStatusCode.Unauthorized) {
          res.status(HttpStatusCode.Unauthorized).render('login', { 
            title: "SSO Salman ITB | Login",
            error: error.message,
            email: email
          });
        } else {
          ErrorMiddleware.handle(res, error);
        }
      }
    };
  }
}

export default ValidationMiddleware;
