import { HttpStatusCode } from '@constants/http.enum';
import { HttpException } from '@exceptions/http.exception';
import { IUserDataToService } from '@interfaces/auth.interface';
import { PrismaClient, User } from '@prisma/client';
import { genSalt, hash } from 'bcryptjs';


export class UserService {
  private userModel = new PrismaClient().user;

  public async getUserById(user_id: number): Promise<User> {
    const user = await this.userModel.findUnique({
      where: {
        user_id: user_id,
      },
    });

    return user;
  }

  public async getUserByEmail(email: string): Promise<User> {
    const user = await this.userModel.findUnique({
      where: {
        email: email,
      },
    });

    return user;
  }

  public async resetPassword(user_id:number, password: string){
    const hashedPassword = await hash(password, await genSalt(10));

    const updatedUser = await this.userModel.update({
      where:{
        user_id:user_id
      },
      data: {
        password:hashedPassword
      }
    });

    return updatedUser;
  }

  public async addUser(userData: User): Promise<IUserDataToService> {
    const errors: Record<string, string[]> = {};

    const isEmailExists = await this.getUserByEmail(userData.email);

    if (isEmailExists) {
      errors.email = ["Email is already exists"];
    }

    const isUserIdExists = await this.getUserById(userData.user_id);

    if (isUserIdExists) {
      errors.user_id = ["User id is already exists"];
    }

    if (Object.keys(errors).length > 0) {
      throw new HttpException(
        HttpStatusCode.Conflict, 
        'Operation failed, please check your request again', 
        errors
      );
    }

    const hashedPassword = await hash(userData.password, await genSalt(10));

    const newUser = await this.userModel.create({
      data: {
        user_id: userData.user_id,
        email: userData.email,
        password: hashedPassword,
        position_id: userData.position_id,
        dept_id: userData.dept_id,
        status: userData.status
      }
    });

    return {
      user_id: newUser.user_id,
      email: newUser.email,
      position_id: newUser.position_id,
      dept_id: newUser.dept_id,
      status: newUser.status
    };
  }
}
